
require('dotenv').config();

const jwt = require('jsonwebtoken');
const msg = require('./language_message');
const multer = require('multer');
const path = require('path');
const CommonModel = require('../model/commonModel');


const storage = multer.diskStorage({
  destination: function (req, file, cb) {
    cb(null, 'assets/images/'); // Save uploaded files to the 'assets/images' directory
  },
  filename: function (req, file, cb) {
    const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9);
    cb(null, file.fieldname + '-' + uniqueSuffix + path.extname(file.originalname));
  }
});

const uploadMiddleware = multer({ storage: storage });
//====================================== Post method start ===========================  
//====================================== Post method End =============================  

//====================================== Get method start ===========================  
const getAllContent = async (req, res) => {
  const data = req.query;
  if(!data ){
      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
      return res.json(record);
  } else if(!data.action){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':2}
      return res.json(record);
  }
   else if(data.action !== 'get_all_content'){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':3}
      return res.json(record);
  } else{
  
  try { 
      const content_arr = await CommonModel.getAllContent();
   
      const record = {'success':true,'msg':msg.msgDataFound,'data':{content_arr:content_arr}}
      return res.json(record);
    
   
} catch (error) {
   console.log("database error key 2");
  const record = {'success':false,'msg':msg.msgServerError,'key':error}
  return res.json(record);
}
  }
};
//====================================== updateContent start =========================== 
const updateContent = async (req, res) => {
  
  if (!req.body) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }
 else {
    
  const data = req.body;
 
  if ((data === '') || !('content' in data) || (data.content === '')  || !('content_id' in data) || (data.content_id === '')) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }else{
       

    const { content_id, content,language_id } = data;
    
        try {
         
          const updateStatus = await CommonModel.updateContent(language_id,content_id,content);
          if (updateStatus === 0) {
            const record = {'success':false,'msg':msg.msgContentUpdateError}
            return res.json(record);
            } else {
            
                const record = {'success':true,'msg':msg.msgContentUpdateSuccess}
                return res.json(record);
              
            }
      } catch (error) {
        console.log("database error key 2",error);
        const record = {'success':false,'msg':msg.msgServerError,'key':error}
        return res.json(record);
      }
      }

  
  }
};
const getAllContactus = async (req, res) => {
  const data = req.query;
  if(!data ){
      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
      return res.json(record);
  } else if(!data.action){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':2}
      return res.json(record);
  }
   else if(data.action !== 'get_all_contactus'){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':3}
      return res.json(record);
  } else{

  
  try { 
      const contactus_arr = await CommonModel.getAllContactus();
   
      const record = {'success':true,'msg':msg.msgDataFound,'data':{contactus_arr:contactus_arr}}
      return res.json(record);
    
   
} catch (error) {
   console.log("database error key 2");
  const record = {'success':false,'msg':msg.msgServerError,'key':error}
  return res.json(record);
}
  }
};
const deleteContactus = async (req, res) => {
  
  if (!req.body) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }
 else {
    
  const data = req.body;
 
  if ((data === '') || !('contact_id' in data) || (data.contact_id === '')) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }else{
       

    const { contact_id } = data;
    
        try {
          const  contact_id = parseInt(Buffer.from(data.contact_id, 'base64').toString()); 
          const checkContactusID = await CommonModel.checkContactus(contact_id);
          if (checkContactusID === 0) {
            const record = {'success':false,'msg':msg.msgContactusNotExist}
            return res.json(record);
            } else {


        try {
         
          const updateStatus = await CommonModel.deleteContactus(contact_id);
          if (updateStatus === 0) {
            const record = {'success':false,'msg':msg.msgContactusDeleteError}
            return res.json(record);
            } else {
            
                const record = {'success':true,'msg':msg.msgContactusDeleteSuccess}
                return res.json(record);
              
            }
      } catch (error) {
        console.log("database error key 2",error);
        const record = {'success':false,'msg':msg.msgServerError,'key':error}
        return res.json(record);
      }
    }
      } catch (error) {
   console.log("database error key 1");
  const record = {'success':false,'msg':msg.msgServerError,'key':error}
  return res.json(record);
}
      }

  
  }
};
//====================================== updateContent start =========================== 
const replyContactus = async (req, res) => {
  
  if (!req.body) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }
 else {
    
  const data = req.body;
 
  if ((data === '') || !('reply_message' in data) || (data.reply_message === '')  || !('subject' in data) || (data.subject === '') || !('contact_id' in data) || (data.contact_id === '') ) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }else{
       

    const { reply_message,subject } = data;
    try {
      const  contact_id = parseInt(Buffer.from(data.contact_id, 'base64').toString()); 
      const checkFaqID = await CommonModel.checkContactus(contact_id);
      if (checkFaqID === 0) {
        const record = {'success':false,'msg':msg.msgContactusNotExist}
        return res.json(record);
        } else {
        try {
         
          const updateStatus = await CommonModel.updateContactus(contact_id,reply_message, subject);
          if (updateStatus === 0) {
            const record = {'success':false,'msg':msg.msgContactusUpdateError}
            return res.json(record);
            } else {
            
                const record = {'success':true,'msg':msg.msgContactusUpdateSuccess}
                return res.json(record);
              
            }
      } catch (error) {
        console.log("database error key 2");
        const record = {'success':false,'msg':msg.msgServerError,'key':error}
        return res.json(record);
      }
    }
  } catch (error) {
console.log("database error key 1");
const record = {'success':false,'msg':msg.msgServerError,'key':error}
return res.json(record);
}
      }

  
  }
};
//============= faq start =====================================
const getAllFaqs = async (req, res) => {
  const data = req.query;
  if(!data ){
      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
      return res.json(record);
  } else if(!data.action){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':2}
      return res.json(record);
  }
   else if(data.action !== 'get_all_faqs'){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':3}
      return res.json(record);
  } else{

  
  try { 
      const faqs_arr = await CommonModel.getAllFaqs();
   
      const record = {'success':true,'msg':msg.msgDataFound,'data':{faqs_arr:faqs_arr}}
      return res.json(record);
    
   
} catch (error) {
   console.log("database error key 2");
  const record = {'success':false,'msg':msg.msgServerError,'key':error}
  return res.json(record);
}
  }
};
//====================================== addFaq start =========================== 
const addFaq = async (req, res) => {
  
  if (!req.body) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }
 else {
    
  const data = req.body;
 
  if ((data === '') || !('question' in data) || (data.question === '')  || !('answer' in data) || (data.answer === '') || !('question_hindi' in data) || (data.question_hindi === '')  || !('answer_hindi' in data) || (data.answer_hindi === '')) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
   return res.json(record);

  }else{
       

    const {faq_type, question, answer,question_hindi,answer_hindi } = data;
    
        try {
         
          const addStatus = await CommonModel.addFaq(faq_type,question, answer,question_hindi,answer_hindi);
          if (addStatus === 0) {
            const record = {'success':false,'msg':msg.msgFaqAddError}
            return res.json(record);
            } else {
            
                const record = {'success':true,'msg':msg.msgFaqAddSuccess}
                return res.json(record);
              
            }
      } catch (error) {
        console.log("database error key 2");
        const record = {'success':false,'msg':msg.msgServerError,'key':error}
        return res.json(record);
      }
      }

  
  }
};
//====================================== updateContent start =========================== 
const updateFaq = async (req, res) => {
  
  if (!req.body) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }
 else {
    
  const data = req.body;
 
  if ((data === '') || !('question' in data) || (data.question === '')  || !('answer' in data) || (data.answer === '') || !('question_hindi' in data) || (data.question_hindi === '')  || !('answer_hindi' in data) || (data.answer_hindi === '') || !('faq_id' in data) || (data.faq_id === '') || !('faq_type' in data) || (data.faq_type === '')) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }else{
       

    const { faq_type,question, answer,question_hindi,answer_hindi } = data;
    try {
      const  faq_id = parseInt(Buffer.from(data.faq_id, 'base64').toString()); 
      const checkFaqID = await CommonModel.checkFaq(faq_id);
      if (checkFaqID === 0) {
        const record = {'success':false,'msg':msg.msgFaqNotExist}
        return res.json(record);
        } else {
        try {
         
          const updateStatus = await CommonModel.updateFaq(faq_type,faq_id,question, answer,question_hindi,answer_hindi);
          if (updateStatus === 0) {
            const record = {'success':false,'msg':msg.msgFaqUpdateError}
            return res.json(record);
            } else {
            
                const record = {'success':true,'msg':msg.msgFaqUpdateSuccess}
                return res.json(record);
              
            }
      } catch (error) {
        console.log("database error key 2");
        const record = {'success':false,'msg':msg.msgServerError,'key':error}
        return res.json(record);
      }
    }
  } catch (error) {
console.log("database error key 1");
const record = {'success':false,'msg':msg.msgServerError,'key':error}
return res.json(record);
}
      }

  
  }
};
//====================================== deleteFaq start =========================== 
const deleteFaq = async (req, res) => {
  
  if (!req.body) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }
 else {
    
  const data = req.body;
 
  if ((data === '') || !('faq_id' in data) || (data.faq_id === '')) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }else{
       

    const { faq_id } = data;
    
        try {
          const  faq_id = parseInt(Buffer.from(data.faq_id, 'base64').toString()); 
          const checkFaqID = await CommonModel.checkFaq(faq_id);
          if (checkFaqID === 0) {
            const record = {'success':false,'msg':msg.msgFaqNotExist}
            return res.json(record);
            } else {


        try {
         
          const updateStatus = await CommonModel.deleteFaq(faq_id);
          if (updateStatus === 0) {
            const record = {'success':false,'msg':msg.msgFaqDeleteError}
            return res.json(record);
            } else {
            
                const record = {'success':true,'msg':msg.msgFaqDeleteSuccess}
                return res.json(record);
              
            }
      } catch (error) {
        console.log("database error key 2",error);
        const record = {'success':false,'msg':msg.msgServerError,'key':error}
        return res.json(record);
      }
    }
      } catch (error) {
   console.log("database error key 1");
  const record = {'success':false,'msg':msg.msgServerError,'key':error}
  return res.json(record);
}
      }

  
  }
};

// const getUserAnalyticalReports = async (req, res) => {
//   const data = req.query;
//   if(!data ){
//       const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
//       return res.json(record);
//   } else if(!data.action){
//     const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':2}
//       return res.json(record);
//   }
//    else if(data.action !== 'get_users_analytical_report'){
//     const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':3}
//       return res.json(record);
//   } else{

  
//   try { 
//     const user_type=1;
//     const month_report_arr = [];
//     const year_report_arr  = [];
//     const month_arr = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];

//     let current_year = new Date().getFullYear();
//     for (let i = 0; i < month_arr.length; i++) {
      
//       let month_text = month_arr[i];
//       let current_month = i + 1;
//       let type='';
//        type='All';
//       let month_user_arr = await userModel.getUserAnalyticalReports('monthly',current_year,current_month,type,user_type);
      
//       type='delete';
//       let month_deleted_user_arr = await userModel.getUserAnalyticalReports('monthly',current_year,current_month,type,user_type);

//       type='active';
//       let month_active_user_arr = await userModel.getUserAnalyticalReports('monthly',current_year,current_month,type,user_type);

//       type='deactive';
//       let month_deactive_user_arr = await userModel.getUserAnalyticalReports('monthly',current_year,current_month,type,user_type);
      
//       month_report_arr.push({ month: month_text, month_user_arr: month_user_arr, month_deleted_user_arr: month_deleted_user_arr, month_active_user_arr: month_active_user_arr, month_deactive_user_arr: month_deactive_user_arr });
//     }
//     let currentYear = new Date().getFullYear();
//     for (let i = 2023; i <= currentYear; i++) {
//       let current_month ='';
//       let current_year = i;
//       let year_text = i;
//       let type='';
//       type='All';
//       let year_user_arr = await userModel.getUserAnalyticalReports('yearly',current_year,current_month,type,user_type);

//       type='delete';
//       let year_deleted_user_arr = await userModel.getUserAnalyticalReports('yearly',current_year,current_month,type,user_type);

//       type='active';
//       let year_active_user_arr = await userModel.getUserAnalyticalReports('yearly',current_year,current_month,type,user_type);

//       type='deactive';
//       let year_deactive_user_arr = await userModel.getUserAnalyticalReports('yearly',current_year,current_month,type,user_type);
      
//       year_report_arr.push({ year: year_text, year_user_arr: year_user_arr, year_deleted_user_arr: year_deleted_user_arr, year_active_user_arr: year_active_user_arr, year_deactive_user_arr: year_deactive_user_arr });

//     }
//   //  const year_arr=[];
//       const record = {'success':true,'msg':msg.msgDataFound,'data':{month_report_arr:month_report_arr,year_report_arr:year_report_arr}}
//       return res.json(record);
    
   
// } catch (error) {
//    console.log("database error key 2");
//   const record = {'success':false,'msg':msg.msgServerError,'key':error}
//   return res.json(record);
// }
//   }
// };

// //====================================== Get method start ===========================  
// const getAllDrivers = async (req, res) => {
//   const data = req.query;
//   if(!data ){
//       const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
//       return res.json(record);
//   } else if(!data.action){
//     const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':2}
//       return res.json(record);
//   }
//    else if(data.action !== 'get_all_drivers'){
//     const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':3}
//       return res.json(record);
//   } else{
  
//   try { 
//       const user_arr = await userModel.getAllDrivers();
   
//       const record = {'success':true,'msg':msg.msgDataFound,'data':{user_arr:user_arr}}
//       return res.json(record);
    
   
// } catch (error) {
//    console.log("database error key 2");
//   const record = {'success':false,'msg':msg.msgServerError,'key':error}
//   return res.json(record);
// }
//   }
// };
// const getAllDeletedDrivers = async (req, res) => {
//   const data = req.query;
//   if(!data ){
//       const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
//       return res.json(record);
//   } else if(!data.action){
//     const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':2}
//       return res.json(record);
//   }
//    else if(data.action !== 'get_all_deleted_drivers'){
//     const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':3}
//       return res.json(record);
//   } else{

  
//   try { 
//       const user_arr = await userModel.getAllDeletedDrivers();
   
//       const record = {'success':true,'msg':msg.msgDataFound,'data':{user_arr:user_arr}}
//       return res.json(record);
    
   
// } catch (error) {
//    console.log("database error key 2");
//   const record = {'success':false,'msg':msg.msgServerError,'key':error}
//   return res.json(record);
// }
//   }
// };

// const getDriverAnalyticalReports = async (req, res) => {
//   const data = req.query;
//   if(!data ){
//       const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
//       return res.json(record);
//   } else if(!data.action){
//     const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':2}
//       return res.json(record);
//   }
//    else if(data.action !== 'get_drivers_analytical_report'){
//     const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':3}
//       return res.json(record);
//   } else{

  
//   try { 
//     const user_type=2;
//     const month_report_arr = [];
//     const year_report_arr  = [];
//     const month_arr = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];

//     let current_year = new Date().getFullYear();
//     for (let i = 0; i < month_arr.length; i++) {
      
//       let month_text = month_arr[i];
//       let current_month = i + 1;
//       let type='';
//        type='All';
//       let month_user_arr = await userModel.getUserAnalyticalReports('monthly',current_year,current_month,type,user_type);
      
//       type='delete';
//       let month_deleted_user_arr = await userModel.getUserAnalyticalReports('monthly',current_year,current_month,type,user_type);

//       type='active';
//       let month_active_user_arr = await userModel.getUserAnalyticalReports('monthly',current_year,current_month,type,user_type);

//       type='deactive';
//       let month_deactive_user_arr = await userModel.getUserAnalyticalReports('monthly',current_year,current_month,type,user_type);
      
//       month_report_arr.push({ month: month_text, month_user_arr: month_user_arr, month_deleted_user_arr: month_deleted_user_arr, month_active_user_arr: month_active_user_arr, month_deactive_user_arr: month_deactive_user_arr });
//     }
//     let currentYear = new Date().getFullYear();
//     for (let i = 2023; i <= currentYear; i++) {
//       let current_month ='';
//       let current_year = i;
//       let year_text = i;
//       let type='';
//       type='All';
//       let year_user_arr = await userModel.getUserAnalyticalReports('yearly',current_year,current_month,type,user_type);

//       type='delete';
//       let year_deleted_user_arr = await userModel.getUserAnalyticalReports('yearly',current_year,current_month,type,user_type);

//       type='active';
//       let year_active_user_arr = await userModel.getUserAnalyticalReports('yearly',current_year,current_month,type,user_type);

//       type='deactive';
//       let year_deactive_user_arr = await userModel.getUserAnalyticalReports('yearly',current_year,current_month,type,user_type);
      
//       year_report_arr.push({ year: year_text, year_user_arr: year_user_arr, year_deleted_user_arr: year_deleted_user_arr, year_active_user_arr: year_active_user_arr, year_deactive_user_arr: year_deactive_user_arr });

//     }
//   //  const year_arr=[];
//       const record = {'success':true,'msg':msg.msgDataFound,'data':{month_report_arr:month_report_arr,year_report_arr:year_report_arr}}
//       return res.json(record);
    
   
// } catch (error) {
//    console.log("database error key 2");
//   const record = {'success':false,'msg':msg.msgServerError,'key':error}
//   return res.json(record);
// }
//   }
// };
// ,getAllDeletedUsers,getUserAnalyticalReports,getAllDrivers,getAllDeletedDrivers,getDriverAnalyticalReports

const oneSignalNotificationSend= async (req, res) => { 
    const axios = require('axios');

//  const {  title, message, jsonData, player_id_arr, language_id } = req.body;
const title = 'Notification test Title';
const message = 'Notification Message';
const jsonData = { key: 'value' };
const player_id_arr = ['42c10294-c102-45d4-bb45-9332d0035e0a', '42c10294-c102-45d4-bb45-9332d0035e0a'];
const language_id = 0;
  const oneSignalAppId = "8903856a-b219-4665-9a75-8b356bd58ce6";
  const oneSignalAuthorization = "ZTAwMTFjZTktYzk3ZS00ZjBjLWI4ZTMtMjg4NDBkMjAxNWFm";

  // Define notification fields
  let fields;
  if (language_id === 0) {
    fields = {
      app_id: oneSignalAppId,
      contents: { en: message },
      headings: { en: title },
      include_player_ids: player_id_arr,
      data: { action_json: jsonData },
      ios_badgeType: 'Increase',
      ios_badgeCount: 1,
      priority: 10
    };
  } else {
    fields = {
      app_id: oneSignalAppId,
      contents: { ar: message },
      headings: { ar: title },
      include_player_ids: player_id_arr,
      data: { action_json: jsonData },
      ios_badgeType: 'Increase',
      ios_badgeCount: 1,
      priority: 10
    };
  }

  try {
    const response = await axios.post('https://onesignal.com/api/v1/notifications', fields, {
        headers: {
          'Content-Type': 'application/json; charset=utf-8',
          'Authorization': `Basic ${oneSignalAuthorization}`
        }
      });
      if (response.status === 200) {
        const record = {'success':true,'msg':response.data}
        return res.json(record);
      } else {

        const record = {'success':false,'error':data}
        return res.json(record);
      }

  } catch (error) {
    console.error('Error:', error);
    const record = { success: false, error: error.message };
    return res.json(record);
  }
}

// Example usage
// oneSignalNotificationSend('Test Title', 'Test Message', { key: 'value' }, ['player_id_1', 'player_id_2'], 0)
//   .then(result => {
//     if (result.success) {
//       console.log('Notification sent successfully:', result.response);
//     } else {
//       console.error('Failed to send notification:', result.error);
//     }
//   })
//   .catch(err => {
//     console.error('Error occurred:', err);
//   });

module.exports = {getAllContent,updateContent,oneSignalNotificationSend,getAllContactus,deleteContactus,replyContactus,addFaq,updateFaq,deleteFaq,getAllFaqs};