
require('dotenv').config();
const jwt = require('jsonwebtoken');
const msg = require('./language_message');
const multer = require('multer');
const path = require('path');
const UserModel = require('../model/userModel');
const CommonModel = require('../model/commonModel');


const storage = multer.diskStorage({
  destination: function (req, file, cb) {
    cb(null, 'assets/images/'); // Save uploaded files to the 'assets/images' directory
  },
  filename: function (req, file, cb) {
    const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9);
    cb(null, file.fieldname + '-' + uniqueSuffix + path.extname(file.originalname));
  }
});

const uploadMiddleware = multer({ storage: storage });
//====================================== Post method start ===========================  
//====================================== Post method End =============================  

//====================================== Get method start ===========================  
const getAllUsers = async (req, res) => {
  const data = req.query;
  if(!data ){
      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
      return res.json(record);
  } else if(!data.action){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':2}
      return res.json(record);
  }
   else if(data.action !== 'get_all_users'){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':3}
      return res.json(record);
  } else{
  
  try { 
      const user_arr = await UserModel.getAllUsers();
   
      const record = {'success':true,'msg':msg.msgDataFound,'data':{user_arr:user_arr}}
      return res.json(record);
    
   
} catch (error) {
   console.log("database error key 2");
  const record = {'success':false,'msg':msg.msgServerError,'key':error}
  return res.json(record);
}
  }
};
const getViewUser = async (req, res) => {
  const data = req.query;
  if(!data ){
      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
      return res.json(record);
  } else if(!data.action){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':2}
      return res.json(record);
  }
   else if(data.action !== 'get_user_details'){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':3}
      return res.json(record);
  } 
  else if(data.user_id === ''){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':3}
      return res.json(record);
  }
  else{
    const  user_id = parseInt(Buffer.from(data.user_id, 'base64').toString());
  try { 

      const user_arr = await UserModel.getViewUser(user_id);
   
      const record = {'success':true,'msg':msg.msgDataFound,'data':{user_arr:user_arr}}
      return res.json(record);
    
   
} catch (error) {
   console.log("database error key 2");
  const record = {'success':false,'msg':msg.msgServerError,'key':error}
  return res.json(record);
}
  }
};
const getViewDriver = async (req, res) => {
  const data = req.query;
  if(!data ){
      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
      return res.json(record);
  } else if(!data.action){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':2}
      return res.json(record);
  }
   else if(data.action !== 'get_driver_details'){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':3}
      return res.json(record);
  } 
  else if(data.user_id === ''){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':3}
      return res.json(record);
  }
  else{
    const  user_id = parseInt(Buffer.from(data.user_id, 'base64').toString());
  try { 

      const user_arr = await UserModel.getViewDriver(user_id);
   
      const record = {'success':true,'msg':msg.msgDataFound,'data':{user_arr:user_arr}}
      return res.json(record);
    
   
} catch (error) {
   console.log("database error key 2");
  const record = {'success':false,'msg':msg.msgServerError,'key':error}
  return res.json(record);
}
  }
};
const getAllDeletedUsers = async (req, res) => {
  const data = req.query;
  if(!data ){
      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
      return res.json(record);
  } else if(!data.action){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':2}
      return res.json(record);
  }
   else if(data.action !== 'get_all_deleted_users'){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':3}
      return res.json(record);
  } else{

  
  try { 
      const user_arr = await UserModel.getAllDeletedUsers();
   
      const record = {'success':true,'msg':msg.msgDataFound,'data':{user_arr:user_arr}}
      return res.json(record);
    
   
} catch (error) {
   console.log("database error key 2");
  const record = {'success':false,'msg':msg.msgServerError,'key':error}
  return res.json(record);
}
  }
};
//====================================== updateApproveUnapproveStatus start =========================== 
const updateApproveUnapproveStatus = async (req, res) => {
  
  if (!req.body) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }
 else {
    
  const data = req.body;
 
  if ((data === '') || !('approve_flag' in data) || (data.approve_flag === '')  || !('user_id' in data) || (data.user_id === '')) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }else{
         
  try {
    const  user_id = parseInt(Buffer.from(data.user_id, 'base64').toString()); 
    const checkUserID = await UserModel.checkUser(user_id);
    if (checkUserID === 0) {
      const record = {'success':false,'msg':msg.msgUserNotExist}
      return res.json(record);
      } else {
        try {
          let approve_flag=0;
         if(data.approve_flag === 1){
          approve_flag=0;
         }else{
          approve_flag=1;
         }
          const updateStatus = await UserModel.updateApproveUnapproveStatus(approve_flag,user_id);
          if (updateStatus === 0) {
            const record = {'success':false,'msg':msg.msgApproveStatusUpdateError}
            return res.json(record);
            } else {
            
                const record = {'success':true,'msg':msg.msgApproveStatusUpdateSuccess,'data':{approve_flag:approve_flag}}
                return res.json(record);
              
            }
      } catch (error) {
        console.log("database error key 2",error);
        const record = {'success':false,'msg':msg.msgServerError,'key':error}
        return res.json(record);
      }
      }
} catch (error) {
  console.log("database error key 3",error);
  const record = {'success':false,'msg':msg.msgServerError,'key':3}
  return res.json(record);
}
  }
  }
};
//====================================== updateActiveDeactiveStatus start =========================== 
const updateActiveDeactiveStatus = async (req, res) => {
  
  if (!req.body) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }
 else {
    
  const data = req.body;
 
  if ((data === '') || !('active_flag' in data) || (data.active_flag === '')  || !('user_id' in data) || (data.user_id === '')) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }else{
         
  try {
    const  user_id = parseInt(Buffer.from(data.user_id, 'base64').toString()); 
    const checkUserID = await UserModel.checkUser(user_id);
    if (checkUserID === 0) {
      const record = {'success':false,'msg':msg.msgUserNotExist}
      return res.json(record);
      } else {
        try {
          let active_flag=0;
         if(data.active_flag === 1){
          active_flag=0;
         }else{
          active_flag=1;
         }
          const updateStatus = await UserModel.updateActiveDeactiveStatus(active_flag,user_id);
          if (updateStatus === 0) {
            const record = {'success':false,'msg':msg.msgActiveDeactiveStatusUpdateError}
            return res.json(record);
            } else {
            
              if(active_flag === 1 ){
                const record = {'success':true,'msg':msg.msgActiveStatusUpdateSuccess,'data':{active_flag:active_flag}}
                return res.json(record);
              }else{
                const record = {'success':true,'msg':msg.msgDeactiveStatusUpdateSuccess,'data':{active_flag:active_flag}}
                return res.json(record);
              }
                
              
            }
      } catch (error) {
        console.log("database error key 2",error);
        const record = {'success':false,'msg':msg.msgServerError,'key':error}
        return res.json(record);
      }
      }
} catch (error) {
  console.log("database error key 3",error);
  const record = {'success':false,'msg':msg.msgServerError,'key':3}
  return res.json(record);
}
  }
  }
};
//====================================== updateTestStatus start =========================== 
const updateTestStatus = async (req, res) => {
  
  if (!req.body) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }
 else {
    
  const data = req.body;
 
  if ((data === '') || !('test_status' in data) || (data.test_status === '')  || !('user_id' in data) || (data.user_id === '')) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }else{
         
  try {
    const  user_id = parseInt(Buffer.from(data.user_id, 'base64').toString()); 
    const checkUserID = await UserModel.checkUser(user_id);
    if (checkUserID === 0) {
      const record = {'success':false,'msg':msg.msgUserNotExist}
      return res.json(record);
      } else {
        try {
          let test_status=0;
         if(data.test_status === 1){
          test_status=0;
         }else{
          test_status=1;
         }
          const updateStatus = await UserModel.updateTestStatus(test_status,user_id);
          if (updateStatus === 0) {
            const record = {'success':false,'msg':msg.msgTestStatusUpdateError}
            return res.json(record);
            } else {
            
              if(test_status === 1 ){
                const record = {'success':true,'msg':msg.msgTestStatusYesUpdateSuccess,'data':{test_status:test_status}}
                return res.json(record);
              }else{
                const record = {'success':true,'msg':msg.msgTestStatusNoUpdateSuccess,'data':{test_status:test_status}}
                return res.json(record);
              }
                
              
            }
      } catch (error) {
        console.log("database error key 2",error);
        const record = {'success':false,'msg':msg.msgServerError,'key':error}
        return res.json(record);
      }
      }
} catch (error) {
  console.log("database error key 3",error);
  const record = {'success':false,'msg':msg.msgServerError,'key':3}
  return res.json(record);
}
  }
  }
};
//====================================== updateTestStatus start =========================== 
const updateDeleteStatus = async (req, res) => {
  
  if (!req.body) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }
 else {
    
  const data = req.body;
 
  if ((data === '') || !('delete_flag' in data) || (data.delete_flag === '')  || !('user_id' in data) || (data.user_id === '')) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }else{
         
  try {
    const  user_id = parseInt(Buffer.from(data.user_id, 'base64').toString()); 
    const checkUserID = await UserModel.checkUser(user_id);
    if (checkUserID === 0) {
      const record = {'success':false,'msg':msg.msgUserNotExist}
      return res.json(record);
      } else {
        try {
          let delete_flag=0;
         if(data.delete_flag === 1){
          delete_flag=0;
         }else{
          delete_flag=1;
         }
          const updateStatus = await UserModel.updateDeleteStatus(delete_flag,user_id);
          if (updateStatus === 0) {
            const record = {'success':false,'msg':msg.msgDeleteStatusUpdateError}
            return res.json(record);
            } else {
            
              if(delete_flag === 1 ){
                const record = {'success':true,'msg':msg.msgDeleteStatusYesUpdateSuccess,'data':{delete_flag:delete_flag}}
                return res.json(record);
              }else{
                const record = {'success':true,'msg':msg.msgDeleteStatusNoUpdateSuccess,'data':{delete_flag:delete_flag}}
                return res.json(record);
              }
                
              
            }
      } catch (error) {
        console.log("database error key 2",error);
        const record = {'success':false,'msg':msg.msgServerError,'key':error}
        return res.json(record);
      }
      }
} catch (error) {
  console.log("database error key 3",error);
  const record = {'success':false,'msg':msg.msgServerError,'key':3}
  return res.json(record);
}
  }
  }
};
//====================================== updateActiveDeactiveStatus start =========================== 
const updateSuspendedActiveStatus = async (req, res) => {
  
  if (!req.body) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }
 else {
    
  const data = req.body;
 
  if ((data === '') || !('active_flag' in data) || (data.active_flag === '')  || !('user_id' in data) || (data.user_id === '')) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }else{
         
  try {
    const  user_id = parseInt(Buffer.from(data.user_id, 'base64').toString()); 
    const checkUserID = await UserModel.checkUser(user_id);
    if (checkUserID === 0) {
      const record = {'success':false,'msg':msg.msgUserNotExist}
      return res.json(record);
      } else {
        try {
          let active_flag=0;
         if(data.active_flag === 1){
          active_flag=0;
         }else{
          active_flag=1;
         }
          const updateStatus = await UserModel.updateActiveDeactiveStatus(active_flag,user_id);
          if (updateStatus === 0) {
            const record = {'success':false,'msg':msg.msgActiveDeactiveStatusUpdateError}
            return res.json(record);
            } else {
            
              if(active_flag === 1 ){
                const record = {'success':true,'msg':msg.msgActiveStatusUpdateSuccess,'data':{active_flag:active_flag}}
                return res.json(record);
              }else{
                const record = {'success':true,'msg':msg.msgDeactiveStatusUpdateSuccess,'data':{active_flag:active_flag}}
                return res.json(record);
              }
                
              
            }
      } catch (error) {
        console.log("database error key 2",error);
        const record = {'success':false,'msg':msg.msgServerError,'key':error}
        return res.json(record);
      }
      }
} catch (error) {
  console.log("database error key 3",error);
  const record = {'success':false,'msg':msg.msgServerError,'key':3}
  return res.json(record);
}
  }
  }
};
 //====================================== updatePassword start =========================== 
const getUserAnalyticalReports = async (req, res) => {
  const data = req.query;
  if(!data ){
      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
      return res.json(record);
  } else if(!data.action){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':2}
      return res.json(record);
  }
   else if(data.action !== 'get_users_analytical_report'){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':3}
      return res.json(record);
  } else{

  
  try { 
    const user_type=1;
    const month_report_arr = [];
    const year_report_arr  = [];
    const month_arr = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];

    let current_year = new Date().getFullYear();
    for (let i = 0; i < month_arr.length; i++) {
      
      let month_text = month_arr[i];
      let current_month = i + 1;
      let type='';
       type='All';
      let month_user_arr = await UserModel.getUserAnalyticalReports('monthly',current_year,current_month,type,user_type);
      
      type='delete';
      let month_deleted_user_arr = await UserModel.getUserAnalyticalReports('monthly',current_year,current_month,type,user_type);

      type='active';
      let month_active_user_arr = await UserModel.getUserAnalyticalReports('monthly',current_year,current_month,type,user_type);

      type='deactive';
      let month_deactive_user_arr = await UserModel.getUserAnalyticalReports('monthly',current_year,current_month,type,user_type);
      
      month_report_arr.push({ month: month_text, month_user_arr: month_user_arr, month_deleted_user_arr: month_deleted_user_arr, month_active_user_arr: month_active_user_arr, month_deactive_user_arr: month_deactive_user_arr });
    }
    let currentYear = new Date().getFullYear();
    for (let i = 2023; i <= currentYear; i++) {
      let current_month ='';
      let current_year = i;
      let year_text = i;
      let type='';
      type='All';
      let year_user_arr = await UserModel.getUserAnalyticalReports('yearly',current_year,current_month,type,user_type);

      type='delete';
      let year_deleted_user_arr = await UserModel.getUserAnalyticalReports('yearly',current_year,current_month,type,user_type);

      type='active';
      let year_active_user_arr = await UserModel.getUserAnalyticalReports('yearly',current_year,current_month,type,user_type);

      type='deactive';
      let year_deactive_user_arr = await UserModel.getUserAnalyticalReports('yearly',current_year,current_month,type,user_type);
      
      year_report_arr.push({ year: year_text, year_user_arr: year_user_arr, year_deleted_user_arr: year_deleted_user_arr, year_active_user_arr: year_active_user_arr, year_deactive_user_arr: year_deactive_user_arr });

    }
  //  const year_arr=[];
      const record = {'success':true,'msg':msg.msgDataFound,'data':{month_report_arr:month_report_arr,year_report_arr:year_report_arr}}
      return res.json(record);
    
   
} catch (error) {
   console.log("database error key 2");
  const record = {'success':false,'msg':msg.msgServerError,'key':error}
  return res.json(record);
}
  }
};

//====================================== Get method start ===========================  
const getAllDrivers = async (req, res) => {
  const data = req.query;
  if(!data ){
      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
      return res.json(record);
  } else if(!data.action){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':2}
      return res.json(record);
  }
   else if(data.action !== 'get_all_drivers'){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':3}
      return res.json(record);
  } else{
  
  try { 
      const user_arr = await UserModel.getAllDrivers();
   
      const record = {'success':true,'msg':msg.msgDataFound,'data':{user_arr:user_arr}}
      return res.json(record);
    
   
} catch (error) {
   console.log("database error key 2");
  const record = {'success':false,'msg':msg.msgServerError,'key':error}
  return res.json(record);
}
  }
};
const getAllDeletedDrivers = async (req, res) => {
  const data = req.query;
  if(!data ){
      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
      return res.json(record);
  } else if(!data.action){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':2}
      return res.json(record);
  }
   else if(data.action !== 'get_all_deleted_drivers'){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':3}
      return res.json(record);
  } else{

  
  try { 
      const user_arr = await UserModel.getAllDeletedDrivers();
   
      const record = {'success':true,'msg':msg.msgDataFound,'data':{user_arr:user_arr}}
      return res.json(record);
    
   
} catch (error) {
   console.log("database error key 2");
  const record = {'success':false,'msg':msg.msgServerError,'key':error}
  return res.json(record);
}
  }
};

const getDriverAnalyticalReports = async (req, res) => {
  const data = req.query;
  if(!data ){
      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
      return res.json(record);
  } else if(!data.action){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':2}
      return res.json(record);
  }
   else if(data.action !== 'get_drivers_analytical_report'){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':3}
      return res.json(record);
  } else{

  
  try { 
    const user_type=2;
    const month_report_arr = [];
    const year_report_arr  = [];
    const month_arr = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];

    let current_year = new Date().getFullYear();
    for (let i = 0; i < month_arr.length; i++) {
      
      let month_text = month_arr[i];
      let current_month = i + 1;
      let type='';
       type='All';
      let month_user_arr = await UserModel.getUserAnalyticalReports('monthly',current_year,current_month,type,user_type);
      
      type='delete';
      let month_deleted_user_arr = await UserModel.getUserAnalyticalReports('monthly',current_year,current_month,type,user_type);

      type='active';
      let month_active_user_arr = await UserModel.getUserAnalyticalReports('monthly',current_year,current_month,type,user_type);

      type='deactive';
      let month_deactive_user_arr = await UserModel.getUserAnalyticalReports('monthly',current_year,current_month,type,user_type);
      
      month_report_arr.push({ month: month_text, month_user_arr: month_user_arr, month_deleted_user_arr: month_deleted_user_arr, month_active_user_arr: month_active_user_arr, month_deactive_user_arr: month_deactive_user_arr });
    }
    let currentYear = new Date().getFullYear();
    for (let i = 2023; i <= currentYear; i++) {
      let current_month ='';
      let current_year = i;
      let year_text = i;
      let type='';
      type='All';
      let year_user_arr = await UserModel.getUserAnalyticalReports('yearly',current_year,current_month,type,user_type);

      type='delete';
      let year_deleted_user_arr = await UserModel.getUserAnalyticalReports('yearly',current_year,current_month,type,user_type);

      type='active';
      let year_active_user_arr = await UserModel.getUserAnalyticalReports('yearly',current_year,current_month,type,user_type);

      type='deactive';
      let year_deactive_user_arr = await UserModel.getUserAnalyticalReports('yearly',current_year,current_month,type,user_type);
      
      year_report_arr.push({ year: year_text, year_user_arr: year_user_arr, year_deleted_user_arr: year_deleted_user_arr, year_active_user_arr: year_active_user_arr, year_deactive_user_arr: year_deactive_user_arr });

    }
  //  const year_arr=[];
      const record = {'success':true,'msg':msg.msgDataFound,'data':{month_report_arr:month_report_arr,year_report_arr:year_report_arr}}
      return res.json(record);
    
   
} catch (error) {
   console.log("database error key 2");
  const record = {'success':false,'msg':msg.msgServerError,'key':error}
  return res.json(record);
}
  }
};

module.exports = {getAllUsers,getViewUser,updateDeleteStatus,updateTestStatus,getAllDeletedUsers,getUserAnalyticalReports,getAllDrivers,getViewDriver,getAllDeletedDrivers,getDriverAnalyticalReports,updateApproveUnapproveStatus,updateActiveDeactiveStatus};