require('dotenv').config();
const mysql = require('mysql');
const moment = require('moment');
const bcrypt = require("bcrypt");

// console.log(process.env.PORT);
const mysqlcon = mysql.createConnection({
    host: process.env.DB_HOST,
    user: process.env.DB_USERNAME,
    database: process.env.DB_NAME,
    password: process.env.DB_PASSWORD
  });

  // Database  connection 
//   mysqlcon.connect((err) => {
//     if (err) {
//       console.error('Database se connect karne mein error:', err);
//       return;
//     }
//     console.log('Database connected!');
//   });
  function handleDisconnect() {
    mysqlcon.connect(function(err) {
        if (err) {
            console.error('Error connecting to MySQL:', err);
            setTimeout(handleDisconnect, 2000); // retry connection after 2 seconds
        } else {
           // console.log('Connected to MySQL server');
        }
    });

    mysqlcon.on('error', function(err) {
        console.error('MySQL error:', err);
        if (err.code === 'PROTOCOL_CONNECTION_LOST') {
            handleDisconnect(); // attempt to reconnect
        } else {
            throw err;
        }
    });
}

handleDisconnect();
module.exports = {

  //================ default   ============================
  async   getPasswordHash(password) {
    const hash = await bcrypt.hash(password, 10);
    return hash;
  },
  //================ check email function start ============================
  
   async  checkUserEmail(data) {
    return new Promise((resolve, reject) => {
        const { email } = data;
        // console.log(data)
        mysqlcon.query('SELECT user_id FROM user_master where delete_flag= 0 and user_type=0 and profile_complete=1 and email = ?',[email], (error, rows) => {
            if (error) {
                console.log('database email check error ',error)
                reject(error);       // Reject the promise with the error
            } else {
                if (rows.length > 0) {
                                     // Email exists, resolve with user_id
                    resolve(rows[0].user_id); // Resolve with user_id
                } else {
                                      // Email does not exist, resolve with 0
                    resolve(0);       // Resolve with 0
                }
            }
        });
    });
    
    },
//========================================== check email and password function start =========================================
   async  checkUserEmailPassword(email,password) {
    return new Promise((resolve, reject) => {
        //console.log('data',data)
       // const { password, email } = data;
        

        mysqlcon.query('SELECT user_id,email,name,user_type,profile_complete,otp_verify,image FROM user_master where delete_flag= 0 and user_type=0 and profile_complete=1 and email =  ? and BINARY password = ? ',[email,password], (error, rows) => {
            if (error) {
                console.log('database email password check error',error);       
                reject(error);       
            } else {
                if (rows.length > 0) {
                               
                    resolve(rows[0].user_id); // Resolve with user_id
                } else {
                                      
                    resolve(0);      
                }
            }
        });
    });
    
    },
//================ check User function start ============================
  
   async  checkUser(data) {
    return new Promise((resolve, reject) => {
        const { user_id } = data;
        // console.log(data)
        mysqlcon.query('SELECT user_id FROM user_master where delete_flag= 0 and user_type=0 and profile_complete=1 and user_id = ?',[user_id], (error, rows) => {
            if (error) {
                console.log('database user check error ')
                reject(error);       // Reject the promise with the error
            } else {
                if (rows.length > 0) {
                                     // Email exists, resolve with user_id
                    resolve(rows[0].user_id); // Resolve with user_id
                } else {
                                      // Email does not exist, resolve with 0
                    resolve(0);       // Resolve with 0
                }
            }
        });
    });
    
    },
//================ update User profile function start ============================
  
   async  updateUserProfile(email,name,mobile,user_id,image) {
    return new Promise((resolve, reject) => {
        const updatetime = moment().format('YYYY-MM-DD HH:mm:ss');

        mysqlcon.query('UPDATE user_master SET name = ?, email = ?,mobile=?,image=?,updatetime=? WHERE delete_flag=0 and  user_id = ? ',[name,email,mobile,image,updatetime,user_id], (error, rows) => {
            if (error) {
                console.log('database user check error ')
                reject(error);       // Reject the promise with the error
            } else {
                if (rows.affectedRows > 0) {
                                     // Email exists, resolve with user_id
                    resolve(rows.affectedRows); // Resolve with user_id
                } else {
                                      // Email does not exist, resolve with 0
                    resolve(0);       // Resolve with 0
                }
            }
        });
    });
    
    },
//================ update  User password function start ============================
  
   async  updateUserPassword(password,user_id) {
    return new Promise((resolve, reject) => {
        const updatetime = moment().format('YYYY-MM-DD HH:mm:ss');

        mysqlcon.query('UPDATE user_master SET password = ?,updatetime=? WHERE delete_flag=0 and  user_id = ? ',[password,updatetime,user_id], (error, rows) => {
            if (error) {
                console.log('database user check error ')
                reject(error);       // Reject the promise with the error
            } else {
                if (rows.affectedRows > 0) {
                                     // Email exists, resolve with user_id
                    resolve(rows.affectedRows); // Resolve with user_id
                } else {
                                      // Email does not exist, resolve with 0
                    resolve(0);       // Resolve with 0
                }
            }
        });
    });
    
    },
//================ check checkUserOldPassword function start ============================
  
   async  checkUserOldPassword(oldpassword,user_id) {
    return new Promise((resolve, reject) => {
   
        mysqlcon.query('SELECT user_id FROM user_master where delete_flag= 0 and user_type=0 and profile_complete=1 and BINARY password = ? and user_id = ?',[oldpassword,user_id], (error, rows) => {
            if (error) {
                console.log('database user check old pass error ')
                reject(error);       // Reject the promise with the error
            } else {
                if (rows.length > 0) {
                                     // Email exists, resolve with user_id
                    resolve(1); // Resolve with user_id
                } else {
                                      // Email does not exist, resolve with 0
                    resolve(0);       // Resolve with 0
                }
            }
        });
    });
    
    },
  
 //========================================== =======================================

   
  };