require('dotenv').config();
const nodemailer = require('nodemailer');

module.exports = {
    async mailSend(email, fromName, subject, mailBody) {
        console.log(process.env.MAIL_USERNAME);
        console.log(process.env.MAIL_PASSWORD);
        console.log(process.env.MAIL_HOST);
        
        // Create a Nodemailer transporter
        let transporter = nodemailer.createTransport({
           
            host: process.env.MAIL_HOST,
            port: process.env.MAIL_PORT,
            secure: process.env.MAIL_SMTPSECURE,
            auth: {
                user: process.env.MAIL_USERNAME,
                pass: process.env.MAIL_PASSWORD
                
            },
            debug: true
        });
        
        let mailOptions = {
            from: process.env.MAIL_FROM,
            to: email,
            subject: subject,
            html: mailBody,
        };

        // Send mail with defined transport object
        try {
            let info = await transporter.sendMail(mailOptions);
            console.log('Message sent: %s', info.messageId);
            return 'yes';
        } catch (error) {
            console.error(error);
            console.log('Error occurred while sending email');
            return 'no';
        }
    },
    
async mailSendWithAttachment(email, fromName, subject, mailBody,filename,path) {
        console.log(process.env.MAIL_USERNAME);
        console.log(process.env.MAIL_PASSWORD);
        console.log(process.env.MAIL_HOST);
        
        // Create a Nodemailer transporter
        let transporter = nodemailer.createTransport({
           
            host: process.env.MAIL_HOST,
            port: process.env.MAIL_PORT,
            secure: process.env.MAIL_SMTPSECURE,
            auth: {
                user: process.env.MAIL_USERNAME,
                pass: process.env.MAIL_PASSWORD
                
            },
            debug: true
        });
        const attachment = {
            filename: filename,
            path: path // URL of the file
        };
        let mailOptions = {
            from: process.env.MAIL_FROM,
            to: email,
            subject: subject,
            html: mailBody,
            attachments: [attachment]
        };

        // Send mail with defined transport object
        try {
            let info = await transporter.sendMail(mailOptions);
            console.log('Message sent: %s', info.messageId);
            return 'yes';
        } catch (error) {
            console.error(error);
            console.log('Error occurred while sending email');
            return 'no';
        }
    },
// Forgot password mail body
async mailBodyData(postData) {
    const date = new Date().getFullYear();
    let body = `
        <!DOCTYPE html>
        <html>
        <head>
            <meta name="viewport" content="width=device-width, initial-scale=1">
            <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
            <title>Welcome to ${postData.appName}</title>
        </head>
        <body style="margin: 0; padding: 0; font-size: 13px; color: #444; font-family: Arial, Helvetica, sans-serif; padding-top: 70px; padding-bottom: 70px;">
            <table cellspacing="0" cellpadding="0" align="center" width="768" class="outer-tbl" style="margin: 0 auto;">
                <tr>
                    <td class="pad-l-r-b" style="background-color: ${postData.borderBackground}; padding: 0 70px 40px;">
                        <table cellpadding="0" cellspacing="0" class="full-wid"></table>
                        <table cellpadding="0" cellspacing="0" style="width: 100%; background-color: #FFFFFF; border-radius: 4px; box-shadow: 0 0 20px #ccc; margin-top: 40px;">
                            <tr>
                                <td>
                                    <table border="0" style="margin: 0; width: 100%;" cellpadding="0" cellspacing="0">
                                        <tr>
                                            <td class="logo" style="padding: 40px 0 30px 0; background-color: #FFFFFF; text-align: center; border-bottom: 1px solid #E1E1E1;">
                                                <img src="${postData.appLogo}" alt="" width="15%" height="18%" style="background-color: white; padding: 5px;">
                                                <h1 style="color: #000000;">${postData.mailHeading}</h1>
                                            </td>
                                        </tr>
                                        <tr><td></td></tr>
                                        <tr>
                                            <td class="content" style="padding: 40px 40px;">
                                                <p style="font-family: Arial, Helvetica, sans-serif; font-size: 15px; color: #333333; margin-top: 0;">
                                                    ${postData.headerGreeting} ${postData.name}</p>
                                                <p style="font-family: Arial, Helvetica, sans-serif; font-size: 15px; color: #333333; margin-top: 0;">
                                                    ${postData.mailContent}</p>
                                                <p style="font-family: Arial, Helvetica, sans-serif; font-size: 15px; color: #333333; margin-top: 0;">
                                                 ${postData.footerGreeting},                  	
                                                </p>
                                                <p style="font-family: Arial, Helvetica, sans-serif; font-size: 15px; color: #333333; margin-top: 0;">
                                                    ${postData.appName}            	
                                                </p>
                                            </td>
                                        </tr>
                                        <tr>
                                            <td  style="background: ${postData.footerBackground}; padding-bottom: 60px;">
                                                <table style="width: 100%;" border="0" cellspacing="0" cellpadding="0" class="full-wid" align="center">
                                                    <tr>
                                                        <td>
                                                            <div style="margin: 0 auto; text-align: center; padding: 0 100px;" class="foot-items">
                                                                <p style="font-family: Arial, Helvetica, sans-serif; font-size: 14px; color: #fbfbfb; margin-top: 40px; line-height: 20px;">
                                                                    &#169; ${date} ${postData.appName}.  ${postData.footerDescription}
                                                                </p>
                                                            </div>
                                                        </td>
                                                    </tr>
                                                </table>
                                            </td>
                                        </tr>
                                    </table>
                                </td>
                            </tr>
                        </table>
                    </td>
                </tr>
            </table>
        </body>
        </html>
    `;
    return body;
},
};


