require('dotenv').config();

const AuthModel = require('../model/authModel');

const CommonModel = require('../model/commonModel');

const jwt = require('jsonwebtoken');

const msg = require('./language_message');

const MailFunctions = require('../common/mailFunctions');



const multer = require('multer');

const path = require('path');



const storage = multer.diskStorage({

  destination: function (req, file, cb) {

    cb(null, 'assets/images/'); // Save uploaded files to the 'webservice/images' directory

  },

  filename: function (req, file, cb) {

    const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9);

    cb(null, file.fieldname + '-' + uniqueSuffix + path.extname(file.originalname));

  }

});



const uploadMiddleware = multer({ storage: storage });

//====================================== Post method start ===========================  

const signIn = async (req, res) => {

    const data = req.body;

    

    if ((data === '') || !('email' in data) || (data.email === '') || !('password' in data) || (data.password === '')) {

      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}

      return res.json(record);



    }{

    try {

      const userId = await AuthModel.checkUserEmail(data);

      if (userId === 0) {

        const record = {'success':false,'msg':msg.msgEmailNotExist}

        return res.json(record);

        } else {

          try {

            const email = data.email;

            const password = await CommonModel.md5(data.password);

            //console.log(password);

            const user_id = await AuthModel.checkUserEmailPassword(email,password);

            if (user_id === 0) {

              const record = {'success':false,'msg':msg.msgEmailPasswordNotExist}

              return res.json(record);

              } else {

               const user_details = await CommonModel.getUserDetails(user_id);

              

                if(user_details !=='NA'){

                  const  email  = user_details.email; 

                }

                  const jwt_secret_key = process.env.JWT_SECRET_KEY;

                  const expiresIn = process.env.JWT_EXPIRES_IN;

                  const user_arr = {user_id:user_id,user_type:0,email:email}

                  const token = jwt.sign(user_arr, jwt_secret_key, { expiresIn: expiresIn });

                  // console.log('token',token)

                  const record = {'success':true,'msg':msg.msgLoginSuccess,'token':token,'data':{user_arr:user_details}}

                  return res.json(record);

                

              }

        } catch (error) {

          console.log("database error key 2");

          const record = {'success':false,'msg':msg.msgServerError,'key':error}

          return res.json(record);

        }

        }

  } catch (error) {

    console.log("database error key 3");

    const record = {'success':false,'msg':msg.msgServerError,'key':error}

    return res.json(record);

  }

}

  };

 //

 

 //====================================== Post  Forgot password start ===========================  

 const forgotPassword = async (req, res) => {

    const data = req.body;

    

    if ((data === '') || !('email' in data) || (data.email === '') ) {

      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}

      return res.json(record);



    }{

    try {

      const userId = await AuthModel.checkUserEmail(data);

      if (userId === 0) {

        const record = {'success':false,'msg':msg.msgEmailNotExist}

        return res.json(record);

        } else {

          try {

            const email = data.email;

            const forgotId = await AuthModel.insertSendEmail(email,userId);

            if (forgotId === 0) {

              const record = {'success':false,'msg':msg.msgPasswordResetLinkSendError}

              return res.json(record);

              } else {

                const userDetails = await CommonModel.getUserDetails(userId);
                let name ='NA';
                let language_id ='0';
                 if(userDetails !== 'NA'){
                    name= userDetails.name;
                    language_id= userDetails.language_id;
                 }
                  
                  const fp_code = Buffer.from(forgotId.toString()).toString('base64');
                  const siteURL= process.env.SITE_URL;
                  const mailEmail = 'surendra.youngdecade@gmail.com';
                  const mailName    =name;
                  const resetPassLink = siteURL+'/resetpassword?uniqcode='+fp_code;
                  const mailSubject = msg.mailSubjectForgotPassword[language_id];
                  const mailHeading = msg.mailHeadingForgotPassword[language_id];
                  const headerGreeting =  msg.mailHeaderGreetingForgotPassword[language_id];
                  const mailContent = msg.mailContentForgotPassword[language_id]+'<a href="'+ resetPassLink + '" style="float: unset; width: 25%; display: block; margin: 26px auto 0; background:'+process.env.FOOTERBACKGROUND+'; text-align: center; vertical-align: middle; user-select: none; border: 1px solid transparent; padding: .375rem .75rem; font-size: .875rem; line-height: 1.5; border-radius: .25rem; transition: color .15s ease-in-out,background-color .15s ease-in-out,border-color .15s ease-in-out,box-shadow .15s ease-in-out; color: #fff;text-decoration: unset;">'+msg.mailResetButtonForgotPassword[language_id]+'</a>';
                 
                  try {
                  const mailFromName = process.env.MAIL_FromName;  
                  const appName = process.env.APP_NAME;
                  const appLogo = process.env.APP_LOGO;
                  const borderBackground = process.env.BORDERBACKGROUND;
                  const footerGreeting =  msg.mailFooterGreeting[language_id];
                  const footerDescription = msg.mailFooterDescription[language_id];
                  const footerBackground = process.env.FOOTERBACKGROUND;

                  const mailBody = await MailFunctions.mailBodyData({
                      appName:appName ,
                      appLogo:appLogo ,
                      borderBackground:borderBackground ,
                      mailHeading: mailHeading,
                      headerGreeting: headerGreeting,
                      name: mailName,
                      mailContent:mailContent,
                      footerGreeting:footerGreeting,
                      footerBackground:footerBackground,
                      footerDescription:footerDescription,
                  });

                  try {

                    const responce = await MailFunctions.mailSend(mailEmail, mailFromName, mailSubject, mailBody);

                    console.log('responce',responce)

                     const record = {'success':true,'msg':msg.msgPasswordResetLinkSendSuccess,'data':{responce:responce}}

                     return res.json(record);

                   

                  } catch (error) {

                    console.log("mail error key 2");

                    const record = {'success':false,'msg':msg.msgServerError,'key':error}

                    return res.json(record);

                  }

                } catch (error) {

                  console.log("mail error key 2");

                  const record = {'success':false,'msg':msg.msgServerError,'key':error}

                  return res.json(record);

                }



                

              }

        } catch (error) {

          console.log("database error key 2");

          const record = {'success':false,'msg':msg.msgServerError,'key':error}

          return res.json(record);

        }

        }

  } catch (error) {

    console.log("database error key 3");

    const record = {'success':false,'msg':msg.msgServerError,'key':error}

    return res.json(record);

  }

}

  };

 //====================================== Post  verifyToken start ===========================  

  

  const verifyToken = async (req, res) => {

    if (!('body' in req)) {

      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}

      return res.json(record);

  

    }else{

    if ((req.headers.token === null )) {

      console.log('token is null')

      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':2}

      return res.json(record);

  

    }else{

    const data = req.body;

    console.log(data);

    // console.log(req.headers.token);

    if ((data === '') || !('user_id' in data) || (!data.user_id) || (!req.headers.token) || !('token' in req.headers) ) {

      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}

      return res.json(record);



    }else{

    const token = req.headers.token;


  const  user_id = parseInt(Buffer.from(data.user_id, 'base64').toString());
   // console.log(data);

    const jwt_secret_key = process.env.JWT_SECRET_KEY;

   

    const user_details = await CommonModel.getUserDetails(user_id);

      const  email  = user_details.email; 

    

    await  jwt.verify( token, jwt_secret_key, (err, decodedToken) => {

        if (err) {

            console.error('Token verification failed:');

            const record = {'success':false,'msg':msg.msgExpiredToken,'key':2}

             return res.json(record);

        } else {

           // console.log('Decoded token:', decodedToken);

            

              if (email !== decodedToken.email) {

                const record = {'success':false,'msg':msg.msgExpiredToken,'key':3}

                return res.json(record);

              }

              else{

                const record = {'success':true,'msg':msg.msgValidToken}

                return res.json(record);

               } 

            }

      

    });

  }

}

}

    };

 //====================================== edit profile start =========================== 

 const editProfile = async (req, res) => {

  

  if (!req.body) {

    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}

    return res.json(record);



  }

 else {

    // console.log('hum',req.file);

  const data = req.body;

  // if (!req.file) {

  //   const image = '';

  //    return res.status(400).send('No file uploaded.');

  // }

  // else{

    

  if ((data === '') || !('email' in data) || (data.email === '') || !('name' in data) || (data.name === '') || !('mobile' in data) || (data.mobile === '') || !('user_id' in data) || (data.user_id === '')) {

    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}

    return res.json(record);



  }else{

         

  try {

    

    const checkUserID = await AuthModel.checkUser(data);

    if (checkUserID === 0) {

      const record = {'success':false,'msg':msg.msgUserNotExist}

      return res.json(record);

      } else {

        try {

          const { email,name,mobile } = data;
  const  user_id = parseInt(Buffer.from(data.user_id, 'base64').toString());
          let image ;

          if(!req.file){

            const user_detailsget = await CommonModel.getUserDetails(user_id);



            image=user_detailsget.image;

          }else{

             image = req.file.filename;



          }

         

          const updateUser = await AuthModel.updateUserProfile(email,name,mobile,user_id,image);

          if (updateUser === 0) {

            const record = {'success':false,'msg':msg.msgProfileUpdateError}

            return res.json(record);

            } else {

             const user_details = await CommonModel.getUserDetails(user_id);

            

              if(user_details !=='NA'){

                const  email  = user_details.email; 

              }

                const jwt_secret_key = process.env.JWT_SECRET_KEY;

                const expiresIn = process.env.JWT_EXPIRES_IN;

                const user_arr = {user_id:user_id,user_type:0,email:email}

                const token = jwt.sign(user_arr, jwt_secret_key, { expiresIn: expiresIn });

                // console.log('token',token)

                const record = {'success':true,'msg':msg.msgProfileUpdateSuccess,'token':token,'data':{user_arr:user_details}}

                return res.json(record);

              

            }

      } catch (error) {

        console.log("database error key 2",error);

        const record = {'success':false,'msg':msg.msgServerError,'key':error}

        return res.json(record);

      }

      }

} catch (error) {

  console.log("database error key 3",error);

  const record = {'success':false,'msg':msg.msgServerError,'key':3}

  return res.json(record);

}

  }

  }

};

 //====================================== updatePassword start =========================== 

 const updatePassword = async (req, res) => {

  

  if (!req.body) {

    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}

    return res.json(record);



  }

 else {

  const data = req.body;

 

  if ((!data) || !('password' in data) || (data.password === '') || !('oldpassword' in data) || (data.oldpassword === '') || !('user_id' in data) || (data.user_id === '')) {

    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}

    return res.json(record);



  }else{

         

  try {

    

    const checkUserID = await AuthModel.checkUser(data);

    if (checkUserID === 0) {

      const record = {'success':false,'msg':msg.msgUserNotExist}

      return res.json(record);

      } else {

        try {

            const  user_id = parseInt(Buffer.from(data.user_id, 'base64').toString());

          const oldpassword = await CommonModel.md5(data.oldpassword);

          const password = await CommonModel.md5(data.password);



           const check_user_old_pass = await AuthModel.checkUserOldPassword(oldpassword,user_id);

          if(check_user_old_pass !== 1 ){

            const record = {'success':false,'msg':msg.msgOldPasswordWrong,'key':1}

            return res.json(record);

          }else{



          const update_user_password = await AuthModel.updateUserPassword(password,user_id);

          if (update_user_password === 0) {

            const record = {'success':false,'msg':msg.msgPasswordUpdateError}

            return res.json(record);

            } else {

           

                const record = {'success':true,'msg':msg.msgPasswordUpdateSuccess}

                return res.json(record);

              

            }

          }

      } catch (error) {

        console.log("database error key 2",error);

        const record = {'success':false,'msg':msg.msgServerError,'key':error}

        return res.json(record);

      }

      }

} catch (error) {

  console.log("database error key 3",error);

  const record = {'success':false,'msg':msg.msgServerError,'key':3}

  return res.json(record);

}

  }

  }

};



 //====================================== updatePassword start =========================== 

 const resetPassword = async (req, res) => {

  

  if (!req.body) {

    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}

    return res.json(record);



  }

 else {

  const data = req.body;

 

  if ((!data) || !('password' in data) || (data.password === '') ||  !('unique_id' in data) || (data.unique_id === '')) {

    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}

    return res.json(record);



  }else{

         

  try {

    

    const checkUserID = await AuthModel.checkUserForgotID(data);

    if (checkUserID === 0) {

      const record = {'success':false,'msg':msg.msgUserNotExist}

      return res.json(record);

      } else {

        try {

          

         const  user_id = checkUserID;

          const password = await CommonModel.md5(data.password);





          const update_user_password = await AuthModel.updateUserPassword(password,user_id);

          if (update_user_password === 0) {

            const record = {'success':false,'msg':msg.msgPasswordUpdateError}

            return res.json(record);

            } else {

      

                const record = {'success':true,'msg':msg.msgPasswordUpdateSuccess}

                return res.json(record);

              

            }

          

      } catch (error) {

        console.log("database error key 2",error);

        const record = {'success':false,'msg':msg.msgServerError,'key':error}

        return res.json(record);

      }

      }

} catch (error) {

  console.log("database error key 3",error);

  const record = {'success':false,'msg':msg.msgServerError,'key':3}

  return res.json(record);

}

  }

  }

};





 //====================================== Post  method end ===========================  

 //====================================== Get method start ===========================  



  const getProfile = async (req, res) => {

    if ((req === '' || !('body' in req) ) ){

      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':4}

      return res.json(record);

    }

    const data = req.query;

//  console.log(req);

    if ((data === '') || !('user_id' in data) || (data.user_id === '') ) {

      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}

      return res.json(record);



    }else{

    try { 

      const  user_id = parseInt(Buffer.from(data.user_id, 'base64').toString());




      const user_details = await CommonModel.getUserDetails(user_id);

      const record = {'success':true,'msg':msg.msgDataFound,'data':{user_arr:user_details}}

      return res.json(record);

  } catch (error) {

    // console.log("database error key 2",error);

    const record = {'success':false,'msg':msg.msgServerError,'key':2}

    return res.json(record);

  }

}

   

    

  };

 

  //====================================== Get method end ===========================  



  module.exports = { getProfile,signIn,forgotPassword,resetPassword,verifyToken,editProfile,uploadMiddleware,updatePassword };