
require('dotenv').config();

const jwt = require('jsonwebtoken');
const msg = require('./language_message');
const multer = require('multer');
const path = require('path');
const CommonFunction = require('../common/commonFunction');

const CommonModel = require('../model/commonModel');


const storage = multer.diskStorage({
  destination: function (req, file, cb) {
    cb(null, 'assets/images/'); // Save uploaded files to the 'assets/images' directory
  },
  filename: function (req, file, cb) {
    const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9);
    cb(null, file.fieldname + '-' + uniqueSuffix + path.extname(file.originalname));
  }
});

const uploadMiddleware = multer({ storage: storage });
//====================================== Post method start ===========================  
//====================================== Post method End =============================  

//====================================== Get method  start dashboard ===========================  
const getAllDashboardData = async (req, res) => {
  const data = req.query;
  if(!data ){
      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
      return res.json(record);
  } else if(!data.action){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':2}
      return res.json(record);
  }
   else if(data.action !== 'get_dashboard_data'){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':3}
      return res.json(record);
  } else{
  
  try { 
      const vehicles = await CommonModel.getAllVehicleCount();
      const clients = await CommonModel.getAllClientCount();
      const drivers = await CommonModel.getAllDriverCount();
      const users = await CommonModel.getAllUserCount();
      const bookings = await CommonModel.getAllBookingCount();
   
      const record = {'success':true,'msg':msg.msgDataFound,'data':{clients:clients,drivers:drivers,users:users,bookings:bookings,vehicles:vehicles}}
      return res.json(record);
    
   
} catch (error) {
   console.log("database error key 2");
  const record = {'success':false,'msg':msg.msgServerError,'key':error}
  return res.json(record);
}
  }
};
//====================================== Get content start ===========================  
const getAllContent = async (req, res) => {
  const data = req.query;
  if(!data ){
      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
      return res.json(record);
  } else if(!data.action){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':2}
      return res.json(record);
  }
   else if(data.action !== 'get_all_content'){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':3}
      return res.json(record);
  } else{
  
  try { 
      const content_arr = await CommonModel.getAllContent();
   
      const record = {'success':true,'msg':msg.msgDataFound,'data':{content_arr:content_arr}}
      return res.json(record);
    
   
} catch (error) {
   console.log("database error key 2");
  const record = {'success':false,'msg':msg.msgServerError,'key':error}
  return res.json(record);
}
  }
};
//====================================== updateContent start =========================== 
const updateContent = async (req, res) => {
  
  if (!req.body) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }
 else {
    
  const data = req.body;
 
  if ((data === '') || !('content' in data) || (data.content === '')  || !('content_id' in data) || (data.content_id === '')) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }else{
       

    const { content_id, content,language_id } = data;
    
        try {
         
          const updateStatus = await CommonModel.updateContent(language_id,content_id,content);
          if (updateStatus === 0) {
            const record = {'success':false,'msg':msg.msgContentUpdateError}
            return res.json(record);
            } else {
            
                const record = {'success':true,'msg':msg.msgContentUpdateSuccess}
                return res.json(record);
              
            }
      } catch (error) {
        console.log("database error key 2",error);
        const record = {'success':false,'msg':msg.msgServerError,'key':error}
        return res.json(record);
      }
      }

  
  }
};
const getAllContactus = async (req, res) => {
  const data = req.query;
  if(!data ){
      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
      return res.json(record);
  } else if(!data.action){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':2}
      return res.json(record);
  }
   else if(data.action !== 'get_all_contactus'){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':3}
      return res.json(record);
  } else{

  
  try { 
      const contactus_arr = await CommonModel.getAllContactus();
   
      const record = {'success':true,'msg':msg.msgDataFound,'data':{contactus_arr:contactus_arr}}
      return res.json(record);
    
   
} catch (error) {
   console.log("database error key 2");
  const record = {'success':false,'msg':msg.msgServerError,'key':error}
  return res.json(record);
}
  }
};
const deleteContactus = async (req, res) => {
  
  if (!req.body) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }
 else {
    
  const data = req.body;
 
  if ((data === '') || !('contact_id' in data) || (data.contact_id === '')) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }else{
       

    const { contact_id } = data;
    
        try {
          const  contact_id = parseInt(Buffer.from(data.contact_id, 'base64').toString()); 
          const checkContactusID = await CommonModel.checkContactus(contact_id);
          if (checkContactusID === 0) {
            const record = {'success':false,'msg':msg.msgContactusNotExist}
            return res.json(record);
            } else {


        try {
         
          const updateStatus = await CommonModel.deleteContactus(contact_id);
          if (updateStatus === 0) {
            const record = {'success':false,'msg':msg.msgContactusDeleteError}
            return res.json(record);
            } else {
            
                const record = {'success':true,'msg':msg.msgContactusDeleteSuccess}
                return res.json(record);
              
            }
      } catch (error) {
        console.log("database error key 2",error);
        const record = {'success':false,'msg':msg.msgServerError,'key':error}
        return res.json(record);
      }
    }
      } catch (error) {
   console.log("database error key 1");
  const record = {'success':false,'msg':msg.msgServerError,'key':error}
  return res.json(record);
}
      }

  
  }
};
//====================================== updateContent start =========================== 
const replyContactus = async (req, res) => {
  
  if (!req.body) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }
 else {
    
  const data = req.body;
 
  if ((data === '') || !('reply_message' in data) || (data.reply_message === '')  || !('subject' in data) || (data.subject === '') || !('contact_id' in data) || (data.contact_id === '') ) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }else{
       

    const { reply_message,subject } = data;
    try {
      const  contact_id = parseInt(Buffer.from(data.contact_id, 'base64').toString()); 
      const checkFaqID = await CommonModel.checkContactus(contact_id);
      if (checkFaqID === 0) {
        const record = {'success':false,'msg':msg.msgContactusNotExist}
        return res.json(record);
        } else {
        try {
         
          const updateStatus = await CommonModel.updateContactus(contact_id,reply_message, subject);
          if (updateStatus === 0) {
            const record = {'success':false,'msg':msg.msgContactusUpdateError}
            return res.json(record);
            } else {
            
                const record = {'success':true,'msg':msg.msgContactusUpdateSuccess}
                return res.json(record);
              
            }
      } catch (error) {
        console.log("database error key 2");
        const record = {'success':false,'msg':msg.msgServerError,'key':error}
        return res.json(record);
      }
    }
  } catch (error) {
console.log("database error key 1");
const record = {'success':false,'msg':msg.msgServerError,'key':error}
return res.json(record);
}
      }

  
  }
};
//============= faq start =====================================
const getAllFaqs = async (req, res) => {
  const data = req.query;
  if(!data ){
      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
      return res.json(record);
  } else if(!data.action){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':2}
      return res.json(record);
  }
   else if(data.action !== 'get_all_faqs'){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':3}
      return res.json(record);
  } else{

  
  try { 
      const faqs_arr = await CommonModel.getAllFaqs();
   
      const record = {'success':true,'msg':msg.msgDataFound,'data':{faqs_arr:faqs_arr}}
      return res.json(record);
    
   
} catch (error) {
   console.log("database error key 2");
  const record = {'success':false,'msg':msg.msgServerError,'key':error}
  return res.json(record);
}
  }
};
//====================================== addFaq start =========================== 
const addFaq = async (req, res) => {
  
  if (!req.body) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }
 else {
    
  const data = req.body;
 
  if ((data === '') || !('question' in data) || (data.question === '')  || !('answer' in data) || (data.answer === '') || !('question_hindi' in data) || (data.question_hindi === '')  || !('answer_hindi' in data) || (data.answer_hindi === '')) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
   return res.json(record);

  }else{
       

    const {faq_type, question, answer,question_hindi,answer_hindi } = data;
    
        try {
         
          const addStatus = await CommonModel.addFaq(faq_type,question, answer,question_hindi,answer_hindi);
          if (addStatus === 0) {
            const record = {'success':false,'msg':msg.msgFaqAddError}
            return res.json(record);
            } else {
            
                const record = {'success':true,'msg':msg.msgFaqAddSuccess}
                return res.json(record);
              
            }
      } catch (error) {
        console.log("database error key 2");
        const record = {'success':false,'msg':msg.msgServerError,'key':error}
        return res.json(record);
      }
      }

  
  }
};
//====================================== updateContent start =========================== 
const updateFaq = async (req, res) => {
  
  if (!req.body) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }
 else {
    
  const data = req.body;
 
  if ((data === '') || !('question' in data) || (data.question === '')  || !('answer' in data) || (data.answer === '') || !('question_hindi' in data) || (data.question_hindi === '')  || !('answer_hindi' in data) || (data.answer_hindi === '') || !('faq_id' in data) || (data.faq_id === '') || !('faq_type' in data) || (data.faq_type === '')) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }else{
       

    const { faq_type,question, answer,question_hindi,answer_hindi } = data;
    try {
      const  faq_id = parseInt(Buffer.from(data.faq_id, 'base64').toString()); 
      const checkFaqID = await CommonModel.checkFaq(faq_id);
      if (checkFaqID === 0) {
        const record = {'success':false,'msg':msg.msgFaqNotExist}
        return res.json(record);
        } else {
        try {
         
          const updateStatus = await CommonModel.updateFaq(faq_type,faq_id,question, answer,question_hindi,answer_hindi);
          if (updateStatus === 0) {
            const record = {'success':false,'msg':msg.msgFaqUpdateError}
            return res.json(record);
            } else {
            
                const record = {'success':true,'msg':msg.msgFaqUpdateSuccess}
                return res.json(record);
              
            }
      } catch (error) {
        console.log("database error key 2");
        const record = {'success':false,'msg':msg.msgServerError,'key':error}
        return res.json(record);
      }
    }
  } catch (error) {
console.log("database error key 1");
const record = {'success':false,'msg':msg.msgServerError,'key':error}
return res.json(record);
}
      }

  
  }
};
//====================================== deleteFaq start =========================== 
const deleteFaq = async (req, res) => {
  
  if (!req.body) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }
 else {
    
  const data = req.body;
 
  if ((data === '') || !('faq_id' in data) || (data.faq_id === '')) {
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
    return res.json(record);

  }else{
       

    const { faq_id } = data;
    
        try {
          const  faq_id = parseInt(Buffer.from(data.faq_id, 'base64').toString()); 
          const checkFaqID = await CommonModel.checkFaq(faq_id);
          if (checkFaqID === 0) {
            const record = {'success':false,'msg':msg.msgFaqNotExist}
            return res.json(record);
            } else {


        try {
         
          const updateStatus = await CommonModel.deleteFaq(faq_id);
          if (updateStatus === 0) {
            const record = {'success':false,'msg':msg.msgFaqDeleteError}
            return res.json(record);
            } else {
            
                const record = {'success':true,'msg':msg.msgFaqDeleteSuccess}
                return res.json(record);
              
            }
      } catch (error) {
        console.log("database error key 2",error);
        const record = {'success':false,'msg':msg.msgServerError,'key':error}
        return res.json(record);
      }
    }
      } catch (error) {
   console.log("database error key 1");
  const record = {'success':false,'msg':msg.msgServerError,'key':error}
  return res.json(record);
}
      }

  
  }
};

//============= getAllNotificationUsers start =====================================
const getAllNotificationUsers = async (req, res) => {
  const data = req.query;
  if(!data ){
      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
      return res.json(record);
  } else if(!data.action){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':2}
      return res.json(record);
  }
   else if(data.action !== 'get_all_notification_users'){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':3}
      return res.json(record);
  } else{

  
  try { 
      const user_arr = await CommonModel.getAllNotificationUsers('all');
   
      const record = {'success':true,'msg':msg.msgDataFound,'data':{user_arr:user_arr}}
      return res.json(record);
    
   
} catch (error) {
   console.log("database error key 2");
  const record = {'success':false,'msg':msg.msgServerError,'key':error}
  return res.json(record);
}
  }
};
//=====================send notification ======================================
const sendNotifications = async (req, res) => {
  // console.log(req);
  if (!req.body) {
    const record = { 'success': false, 'msg': msg.msgAllFieldReqired, 'key': 1 }
    return res.json(record);

  }
  else {

    const data = req.body;

    if ((data === '') || !('action' in data) || (data.message === '') || !('message' in data) || (data.subject === '') || !('subject' in data) || (data.userType === '') || !('userType' in data)) {
      const record = { 'success': false, 'msg': msg.msgAllFieldReqired, 'key': 2, 'data': data }
      return res.json(record);

    } else {
      const { subject, userType, select_arr } = data;
      const send_message = data.message;




      try {

        let user_arr = [];

        if (userType === 'all') {
          user_arr = await CommonModel.getAllNotificationUsers(userType);

        } else {
          let selectArr = data.select_arr;
          if (selectArr.includes('0')) {
            user_arr = await CommonModel.getAllNotificationUsers(userType);
          }
          else {
            user_arr = selectArr.map(async user_id => ({ user_id: user_id, user_type: userType }));
          }

        }
        let successArr = [];
        user_arr.forEach(async userPromise => {
          try {
            let notstatus = '';
            let notificationArr = [];
            let user = await userPromise;

            const user_id = parseInt(Buffer.from(user.user_id, 'base64').toString());
            const user_type = user.user_type;
            const user_id_notification = 1;
            const other_user_id_notification = user_id;
            const action = 'broadcast';
            const action_id = '';
            const title1 = subject;
            const message1 = send_message;
            const title = [title1, title1, title1, title1];
            const message = [message1, message1, message1, message1];
            const action_data = { 'user_id': user_id_notification, 'other_user_id': other_user_id_notification, 'action_id': action_id, 'action': action };

            const notificationArrCheck = await CommonFunction.getNotificationArrSingle(user_id_notification, other_user_id_notification, action, action_id, title, message, action_data);
            if (notificationArrCheck !== 'NA') {
              notificationArr.push(notificationArrCheck);
            }
            if (user_type === 'user') {

              if (notificationArr.length !== 0) {
                notstatus = await CommonFunction.oneSignalNotificationSendCall(notificationArr);
                await successArr.push(notstatus);
              }
            }
            else if (user_type === 'driver') {
              if (notificationArr.length !== 0) {
                notstatus = await CommonFunction.oneSignalNotificationSendCallDriver(notificationArr);
                await successArr.push(notstatus);
              }
            }
            else if (user_type === 'client') {
              if (notificationArr.length !== 0) {
                notstatus = await CommonFunction.oneSignalNotificationSendCallDriver(notificationArr);
                await successArr.push(notstatus);
              }
            }
          } catch (error) {
            // Handle errors if any
          }
        });

        const record = { 'success': true, 'msg': msg.msgNotificationSendSuccess, 'successArr': successArr }
        return res.json(record);


      } catch (error) {
        console.log("database error key 2");
        const record = { 'success': false, 'msg': msg.msgServerError, 'key': error }
        return res.json(record);
      }
    }


  }
};
//============= getAllNotifications start =====================================
const getNotifications = async (req, res) => {
  const data = req.query;
  if(!data ){
      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}
      return res.json(record);
  } else if(!data.action){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':2}
      return res.json(record);
  }
   else if(data.action !== 'get_all_notifications'){
    const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':3}
      return res.json(record);
  } else{

  
  try { 
      const notification_arr = await CommonModel.getNotifications();
   
      const record = {'success':true,'msg':msg.msgDataFound,'data':{notification_arr:notification_arr}}
      return res.json(record);
    
   
} catch (error) {
   console.log("database error key 2");
  const record = {'success':false,'msg':msg.msgServerError,'key':error,'kes':11}
  return res.json(record);
}
  }
};

const oneSignalNotificationSend= async (req, res) => { 
    const axios = require('axios');

//  const {  title, message, jsonData, player_id_arr, language_id } = req.body;
const title = 'Notification test Title';
const message = 'Notification Message';
const jsonData = { key: 'value' };
const player_id_arr = ['42c10294-c102-45d4-bb45-9332d0035e0a', '42c10294-c102-45d4-bb45-9332d0035e0a'];
const language_id = 0;
  const oneSignalAppId = "8903856a-b219-4665-9a75-8b356bd58ce6";
  const oneSignalAuthorization = "ZTAwMTFjZTktYzk3ZS00ZjBjLWI4ZTMtMjg4NDBkMjAxNWFm";

  // Define notification fields
  let fields;
  if (language_id === 0) {
    fields = {
      app_id: oneSignalAppId,
      contents: { en: message },
      headings: { en: title },
      include_player_ids: player_id_arr,
      data: { action_json: jsonData },
      ios_badgeType: 'Increase',
      ios_badgeCount: 1,
      priority: 10
    };
  } else {
    fields = {
      app_id: oneSignalAppId,
      contents: { ar: message },
      headings: { ar: title },
      include_player_ids: player_id_arr,
      data: { action_json: jsonData },
      ios_badgeType: 'Increase',
      ios_badgeCount: 1,
      priority: 10
    };
  }

  try {
    const response = await axios.post('https://onesignal.com/api/v1/notifications', fields, {
        headers: {
          'Content-Type': 'application/json; charset=utf-8',
          'Authorization': `Basic ${oneSignalAuthorization}`
        }
      });
      if (response.status === 200) {
        const record = {'success':true,'msg':response.data}
        return res.json(record);
      } else {

        const record = {'success':false,'error':data}
        return res.json(record);
      }

  } catch (error) {
    console.error('Error:', error);
    const record = { success: false, error: error.message };
    return res.json(record);
  }
}

// Example usage
// oneSignalNotificationSend('Test Title', 'Test Message', { key: 'value' }, ['player_id_1', 'player_id_2'], 0)
//   .then(result => {
//     if (result.success) {
//       console.log('Notification sent successfully:', result.response);
//     } else {
//       console.error('Failed to send notification:', result.error);
//     }
//   })
//   .catch(err => {
//     console.error('Error occurred:', err);
//   });

module.exports = {getAllDashboardData,getAllContent,updateContent,oneSignalNotificationSend,getAllContactus,deleteContactus,replyContactus,addFaq,updateFaq,deleteFaq,getAllFaqs,getNotifications,getAllNotificationUsers,sendNotifications};