require('dotenv').config();
const stripePublishableKey = process.env.STRIPE_PUBLISHABLE_KEY;
const stripeSecretKey = process.env.STRIPE_SECRET_KEY;
const returnUrl = process.env.RETURN_URL;
const cancelUrl = process.env.CANCEL_URL;


const AuthModel = require('../model/authModel');

const CommonModel = require('../model/commonModel');

const jwt = require('jsonwebtoken');

const msg = require('./language_message');

const MailFunctions = require('../common/mailFunctions');



const multer = require('multer');

const path = require('path');



const storage = multer.diskStorage({

  destination: function (req, file, cb) {

    cb(null, 'assets/images/'); // Save uploaded files to the 'webservice/images' directory

  },

  filename: function (req, file, cb) {

    const uniqueSuffix = Date.now() + '-' + Math.round(Math.random() * 1E9);

    cb(null, file.fieldname + '-' + uniqueSuffix + path.extname(file.originalname));

  }

});



const uploadMiddleware = multer({ storage: storage });

//====================================== Get method start ===========================  



  const addCard = async (req, res) => {

    if ((req === '' || !('body' in req) ) ){

      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':4}

      return res.json(record);

    }

    const data = req.query;

//  console.log(req);

    if ((data === '') || !('user_id' in data) || (data.user_id === '') ) {

      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}

      return res.json(record);



    }else{

    try { 

      const  user_id = parseInt(Buffer.from(data.user_id, 'base64').toString());




      const user_details = await CommonModel.getUserDetails(user_id);

      const record = {'success':true,'msg':msg.msgDataFound,'data':{user_arr:user_details}}

      return res.json(record);

  } catch (error) {

    // console.log("database error key 2",error);

    const record = {'success':false,'msg':msg.msgServerError,'key':2}

    return res.json(record);

  }

}

   

    

  };
  const paymentUrl = async (req, res) => {

    if ((req === '' || !('body' in req) ) ){

      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':4}

      return res.json(record);

    }

    const data = req.query;

//  console.log(req);

    if ((data === '') || !('user_id' in data) || (data.user_id === '') ) {

      const record = {'success':false,'msg':msg.msgAllFieldReqired,'key':1}

      return res.json(record);



    }else{

    try { 

      const  user_id = parseInt(Buffer.from(data.user_id, 'base64').toString());




      const user_details = await CommonModel.getUserDetails(user_id);

      const record = {'success':true,'msg':msg.msgDataFound,'data':{user_arr:user_details}}

      return res.json(record);

  } catch (error) {

    // console.log("database error key 2",error);

    const record = {'success':false,'msg':msg.msgServerError,'key':2}

    return res.json(record);

  }

}

   

    

  };

 

  //====================================== Get method end ===========================  
  
  module.exports = { paymentUrl,addCard };